<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Robert Heel <rheel@1drop.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 * Hint: use extdeveval to insert/update function index above.
 */

require_once(PATH_tslib.'class.tslib_pibase.php');
require_once(t3lib_extMgm::extPath('ods_osm').'class.tx_odsosm_div.php');


/**
 * Plugin 'Openstreetmap' for the 'ods_osm' extension.
 *
 * @author	Robert Heel <rheel@1drop.de>
 * @package	TYPO3
 * @subpackage	tx_odsosm
 */
class tx_odsosm_pi1 extends tslib_pibase {
	var $prefixId      = 'tx_odsosm_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_odsosm_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'ods_osm';	// The extension key.
	var $uploadPath    = 'uploads/tx_odsosm/';
	var $pi_checkCHash = true;

	var $escape_js=array(
		"\r\n"=>"<br />",
		"\n"=>"<br />",
		"\r"=>"<br />",
		"'"=>"\'",
		'"'=>'\"',
	);

	var $config;
	var $hooks;
	var $lats=array();
	var $lons=array();
	var $markers;

	function init($conf){
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->pi_initPIflexForm(); // Init FlexForm configuration for plugin

		$this->hooks=array();
		if(is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['ods_osm']['class.tx_odsosm_pi1.php'])){
			foreach($GLOBALS['TYPO3_CONF_VARS']['EXTCONF']['ods_osm']['class.tx_odsosm_pi1.php'] as $classRef){
				$this->hooks[]=&t3lib_div::getUserObj($classRef);
			}
		}

		/* --------------------------------------------------
			Configuration (order of priority)
			- FlexForm
			- TypoScript
			- Extension
		-------------------------------------------------- */

		$flex=array();
		$options=array('height','lat','layer','lon','marker','mouse_navigation','show_layerswitcher','show_pan_zoom','show_popups','static','static_layer','width','zoom');
		foreach($options as $option){
			$value=$this->pi_getFFvalue($this->cObj->data['pi_flexform'],$option,'sDEF');
			if($value){
				switch($option){
					case 'marker':
						$flex[$option]=$this->splitGroup($value,'tt_address');
					break;
					default:
						$flex[$option]=$value;
					break;
				}
			}
		}
		$this->config=array_merge((array)unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['ods_osm']),$conf,$flex);
		if(!is_array($this->config['marker'])) $this->config['marker']=array();
		if(is_array($conf['marker.'])){
			foreach($conf['marker.'] as $name=>$value){
				if(!empty($value)){
					if(!is_array($this->config['marker'][$name])) $this->config['marker'][$name]=array();
					$this->config['marker'][$name]=$this->config['marker'][$name]+explode(',',$value);
				}
			}
		}

		$this->config['layer']=explode(',',$this->config['layer']);

		if(is_numeric($this->config['height'])) $this->config['height'].='px';
		if(is_numeric($this->config['width'])) $this->config['width'].='px';

		if($this->config['show_layerswitcher']){
			$this->config['layers_visible']=array();
		}else{
			$this->config['layers_visible']=$this->config['layer'];
		}

		if($this->config['external_control']){
			if(t3lib_div::_GP('lon')) $this->config['lon']=t3lib_div::_GP('lon');
			if(t3lib_div::_GP('lat')) $this->config['lat']=t3lib_div::_GP('lat');
			if(t3lib_div::_GP('zoom')) $this->config['zoom']=t3lib_div::_GP('zoom');
			if(t3lib_div::_GP('layers')) $this->config['layers_visible']=explode(',',t3lib_div::_GP('layers'));
			if(t3lib_div::_GP('records')) $this->config['marker']=$this->splitGroup(t3lib_div::_GP('records'),'tt_address');
		}

		$this->config['id']='osm_'.$this->cObj->data['uid'];
		$this->config['marker']=$this->extractGroup($this->config['marker']);
	}

	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content, $conf) {
		$this->init($conf);

		if($this->config['marker'] || $this->config['no_marker']){
			if($this->config['static']){
				$content=$this->getStaticMap();
			}else{
				$content=$this->getDynamicMap();
			}
		}

		return $this->pi_wrapInBaseClass($content);
	}

	function splitGroup($group,$default=''){
		$groups=explode(',',$group);
		foreach($groups as $group){
			$item=t3lib_div::revExplode('_',$group,2);
			if(count($item)==1){
				$record_ids[$default][]=$item[0];
			}else{
				$record_ids[$item[0]][]=$item[1];
			}
		}
		return($record_ids);
	}

	function extractGroup($record_ids){
		// get pages
		if(!empty($record_ids['pages'])){
			$tables=array('fe_users','fe_groups','tt_address','tt_address_group','tx_odsosm_track');
			$pids=implode(',',$record_ids['pages']);
			foreach($tables as $table){
				$res=$GLOBALS['TYPO3_DB']->exec_SELECTquery('uid',$table,'pid IN ('.$pids.')'.$this->cObj->enableFields($table));
				while($row=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)){
					$record_ids[$table][]=$row['uid'];
				}
			}
		}

		// get records
		$records=array();
		foreach($record_ids as $table=>$items){
			foreach($items as $item){
				switch($table){
					case 'fe_groups':
						$res=$GLOBALS['TYPO3_DB']->exec_SELECTquery('*','fe_groups','uid='.intval($item).$this->cObj->enableFields('fe_groups'));
						$group=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
						if($group){
							$res=$GLOBALS['TYPO3_DB']->exec_SELECTquery('*','fe_users','FIND_IN_SET("'.intval($item).'",usergroup)'.$this->cObj->enableFields('fe_users'));
							while($row=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)){
								$records['fe_users'][$row['uid']]=$row;
								$records['fe_users'][$row['uid']]['group_title']=$group['title'];
								$records['fe_users'][$row['uid']]['group_description']=$group['description'];
								$records['fe_users'][$row['uid']]['tx_odsosm_marker']=$group['tx_odsosm_marker'];
							}
						}
					break;
					case 'tt_address_group':
						$res=$GLOBALS['TYPO3_DB']->exec_SELECT_mm_query('tt_address.*,tt_address_group.title AS group_title,tt_address_group.description AS group_description,tt_address_group.tx_odsosm_marker','tt_address','tt_address_group_mm','tt_address_group','AND tt_address_group.uid='.intval($item).$this->cObj->enableFields('tt_address').$this->cObj->enableFields('tt_address_group'));
						while($row=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)){
							$records['tt_address'][$row['uid']]=$row;
						}
					break;
					default:
						$res=$GLOBALS['TYPO3_DB']->exec_SELECTquery('*',$table,'uid='.intval($item).$this->cObj->enableFields($table));
						$row=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
						if($row) $records[$table][intval($item)]=$row;
					break;
				}
			}
		}

		// Hook to change records
		foreach($this->hooks as $hook){
			if(method_exists($hook,'changeRecords')){
				$hook->changeRecords($records,$record_ids,$this);
			}
		}

		// Get marker records
		$this->markers=$GLOBALS['TYPO3_DB']->exec_SELECTgetRows('*','tx_odsosm_marker','1'.$this->cObj->enableFields('tx_odsosm_marker'),'','','','uid');

		// get lon&lat
		foreach($records as $table=>$items){
			foreach($items as $uid=>$row){
				switch($table){
					case 'fe_users':
					case 'tt_address':
						if($row['tx_odsosm_lon']){
							$this->lons[]=floatval($row['tx_odsosm_lon']);
							$this->lats[]=floatval($row['tx_odsosm_lat']);
						}else{
							unset($records[$table][$uid]);
						}
					break;
					case 'tx_odsosm_track':
					case 'tx_odsosm_vector':
						if($row['min_lon']){
							$this->lons[]=floatval($row['min_lon']);
							$this->lats[]=floatval($row['min_lat']);
							$this->lons[]=floatval($row['max_lon']);
							$this->lats[]=floatval($row['max_lat']);
						}else{
							unset($records[$table][$uid]);
						}
					break;
				}
			}
		}

		// No markers
		if(count($this->lons)==0){
			if($this->config['no_marker']==1){
				$this->lons[]=0;
				$this->lats[]=0;
				$this->config['zoom']=1;
			}
		}

		return($records);
	}

	function getDynamicMap(){
		/* ==================================================
			Marker
		================================================== */
		// Generate javascript calls
		$jsMarker='';
		$group_titles=array();
		$local_cObj = t3lib_div::makeInstance('tslib_cObj');
		foreach($this->config['marker'] as $table=>$items){
			foreach($items as $item){
				switch($table){
					case 'fe_users':
					case 'tt_address':
						$local_cObj->start($item,$table);
						$text=$this->config['show_popups'] ? $local_cObj->cObjGetSingle($this->config['popup.'][$table],$this->config['popup.'][$table.'.']) : '';
						if($item['tx_odsosm_marker'] && is_array($this->markers[$item['tx_odsosm_marker']])){
							$marker=$this->markers[$item['tx_odsosm_marker']];
							$icon=$GLOBALS['TSFE']->absRefPrefix.'uploads/tx_odsosm/'.$marker['icon'];
						}else{
							$marker=array('size_x'=>21,'size_y'=>25,'offset_x'=>-11,'offset_y'=>-25);
							$icon=$GLOBALS['TSFE']->absRefPrefix.t3lib_extMgm::siteRelPath('ods_osm').'res/marker.png';
						}
						if(!in_array($item['group_title'], $group_titles)) {
							$group_titles[]=$item['group_title'];
							$jsMarker.="var layerMarkers_".array_search($item['group_title'],$group_titles)."=new OpenLayers.Layer.Markers('<img src=\"".$icon."\" /> ".$item['group_title']."');\n";
						}
						$jsMarker.="mapMarker(".$this->config['id'].",layerMarkers_".array_search($item['group_title'],$group_titles).",".$item['tx_odsosm_lat'].",".$item['tx_odsosm_lon'].",'".$icon."',".$marker['size_x'].",".$marker['size_y'].",".$marker['offset_x'].",".$marker['offset_y'].",'".strtr($text,$this->escape_js)."',".intval($this->config['show_popups']).");\n";
					break;
					case 'tx_odsosm_track':
						$jsMarker.="mapGpx(".$this->config['id'].",'".$GLOBALS['TSFE']->absRefPrefix.'uploads/tx_odsosm/'.$item['file']."','".$item['title']."','".$item['color']."',".$item['width'].");\n";
					break;
					case 'tx_odsosm_vector':
						$jsMarker.="mapVector(".$this->config['id'].",'".$item['title']."',".$item['data'].");\n";
					break;
				}
			}
		}

		/* ==================================================
			Layers
		================================================== */
		$layers2=$GLOBALS['TYPO3_DB']->exec_SELECTgetRows('*','tx_odsosm_layer','uid IN ('.implode(',',$this->config['layer']).')'.$this->cObj->enableFields('tx_odsosm_layer'),'','','','uid');
		// same order as 'layer'
		$layers=array();
		foreach($this->config['layer'] as $key){
			$layers[$key]=$layers2[$key];
		}
		// set visible flag
		foreach($this->config['layers_visible'] as $key){
			if($layers[$key]) $layers[$key]['visible']=true;
		}

		// Marker layer
		foreach($group_titles as $key=>$title) {
			$jsMarkerLayer.= $this->config['id'].".addLayer(layerMarkers_".$key.");\n";
		}

		/* ==================================================
			Map center
		================================================== */
		if($this->config['lon']==0){
			$lon=array_sum($this->lons)/count($this->lons);
			$lat=array_sum($this->lats)/count($this->lats);
		}else{
			$lon=$this->config['lon'];
			$lat=$this->config['lat'];
		}
		$zoom=$this->config['zoom'];

		/* ==================================================
			Map header
		================================================== */
 		tx_odsosm_div::getMapCore();
		$GLOBALS['TSFE']->JSeventFuncCalls['onload'][] = "create_".$this->config['id']."();";

		$GLOBALS['TSFE']->getPageRenderer()->addJsInlineCode($this->config['id'],"
			var ".$this->config['id']."; //complex object of type OpenLayers.Map
			function create_".$this->config['id']."(){
				".tx_odsosm_div::getMapMain()."
				".tx_odsosm_div::getMainLayers($layers)."
				mapCenter(".$this->config['id'].",".$lat.",".$lon.",".$zoom.");
				".$jsMarker."
				".$jsMarkerLayer."
			}
		");

		/* ==================================================
			Map container
		================================================== */
		$content='<div style="width:'.$this->config['width'].';height:'.$this->config['height'].';" id="'.$this->config['id'].'"></div>';
		if($this->config['layerswitcher.']['div']) $content.='<div id="'.$this->config['id'].'_layerswitcher" class="olControlLayerSwitcher"></div>';

		return($content);
	}

	function getStaticMap(){
		foreach($this->config['marker'] as $table=>$items){
			foreach($items as $item){
				switch($table){
					case 'fe_users':
					case 'tt_address':
						if($item['tx_odsosm_marker'] && is_array($this->markers[$item['tx_odsosm_marker']])){
							$marker=$this->markers[$item['tx_odsosm_marker']];
							$icon='uploads/tx_odsosm/'.$marker['icon'];
						}else{
							$marker=array('size_x'=>21,'size_y'=>25,'offset_x'=>-11,'offset_y'=>-25);
							$icon='EXT:ods_osm/res/marker.png';
						}
					break 3;
				}
			}
		}

		$markerUrl=array(
			'###lat###'=>floatval($item['tx_odsosm_lat']),
			'###lon###'=>floatval($item['tx_odsosm_lon']),
			'###zoom###'=>intval($this->config['zoom']),
			'###width###'=>intval($this->config['width']),
			'###height###'=>intval($this->config['height']),
		);

		$res=$GLOBALS['TYPO3_DB']->exec_SELECTquery('static_url','tx_odsosm_layer','uid='.$this->config['static_layer'].$this->cObj->enableFields('tx_odsosm_layer'));
		$layer=$GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
		$url=strtr($layer['static_url'],$markerUrl);
		$filename=$this->uploadPath.'map/'.md5($url).'.png';

		// Cache image
		$cache=false;
		if(file_exists($filename)) $cache=filectime($filename)>time()-7*24*60*60;
		if(!$cache){
			$image=file_get_contents($url);
			if($image) file_put_contents($filename,$image);
		}

		// Generate image tag
//		$config['file'] = $filename;
		$config['file']='GIFBUILDER';
		$config['file.']=array(
			'format'=>'png',
			'XY'=>'[10.w],[10.h]',
			'10'=>'IMAGE',
			'10.'=>array(
				'file'=>$filename,
			),
			'20'=>'IMAGE',
			'20.'=>array(
				'offset'=>($this->config['width']/2+$marker['offset_x']).','.($this->config['height']/2+$marker['offset_y']),
				'file'=>$icon,
			),
		);
		$content=$this->cObj->IMAGE($config);
		return($content);
	}
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ods_osm/pi1/class.tx_odsosm_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ods_osm/pi1/class.tx_odsosm_pi1.php']);
}

?>